# %%
import matplotlib.pyplot as plt
import matplotlib.font_manager as fm
import matplotlib.gridspec as gridspec
from highlight_text import ax_text, fig_text
import pandas as pd

from PIL import Image
import urllib
import os

# --- Use this only if you have already downloaded fonts into your
# --- local directory.

# Add pretty fonts

font_path = r"" #Set the path to where the fonts are located

for x in os.listdir(font_path):
    for y in os.listdir(f"{font_path}/{x}"):
        if y.split(".")[-1] == "ttf":
            fm.fontManager.addfont(f"{font_path}/{x}/{y}")
            try:
                fm.FontProperties(weight=y.split("-")[-1].split(".")[0].lower(), fname=y)
            except Exception as e:
                print(f"Font {y} could not be added.")
                continue


# --- Reading the data

df = pd.read_csv("data/mls_set_piece_share_06112022.csv", index_col = 0)
df = df.sort_values(by = "goal_share", ascending = False)

# --- We define a function to plot the battery charts

def plot_battery_chart(data, team_id, ax, legend = False):
    '''
    This function plots the share of xG and goals that were created
    from set-pieces and corners in the MLS.

    Args:
        data (DataFrame): the pandas DataFrame with the data.
        team_id (int): the Fotmob team id.
        ax (object): the matplotlib axes in which to draw the plot.
    '''

    goal_share = data[data["team_id"] == team_id]["goal_share"].iloc[0]
    xg_share = data[data["team_id"] == team_id]["xg_share"].iloc[0]

    # Hatch parameters
    plt.rcParams['hatch.linewidth'] = 0.75
    plt.rcParams['hatch.color'] = "lightgrey"

    # Goal share

    if legend:
        goal_text = f"{goal_share:.1%} goals"
        xg_text = f"{xg_share:.1%} of xG"
    else:
        goal_text = f"{goal_share:.1%}"
        xg_text = f"{xg_share:.1%}"      

    height = .1

    ax.barh([1], [1], height = height, facecolor = "none", ec = "black", zorder = 3)
    ax.barh([1], [goal_share], height = height, color = "#74959A")
    ax.barh([1], [1 - goal_share], height = height, left = goal_share, facecolor = "none", hatch = "////////")
    ax.annotate(
        xy = (goal_share, 1),
        xytext = (20, 0),
        textcoords = "offset points",
        text = goal_text,
        ha = "center",
        va = "center",
        size = 5,
        weight = "bold",
        family = "DM Sans",
        color = "#74959A"
    )

    ax.barh([.85], [1], height = height, facecolor = "none", ec = "black", zorder = 3)
    ax.barh([.85], [xg_share], height = height, color = "#495371")
    ax.barh([.85], [1 - xg_share], height = height, left = xg_share, facecolor = "none", hatch = "////////")
    ax.annotate(
        xy = (xg_share, .85),
        xytext = (20, 0),
        textcoords = "offset points",
        text = xg_text,
        ha = "center",
        va = "center",
        size = 5,
        weight = "bold",
        family = "DM Sans",
        color = "#495371"
    )

    ax.set_axis_off()


# --- We do the final chart.

fig = plt.figure(figsize=(7, 11), dpi = 200)
nrows = 14
ncols = 4
gspec = gridspec.GridSpec(
    ncols=ncols, nrows=nrows, figure=fig, 
    height_ratios = [(1/nrows)*2. if x % 2 != 0 else (1/nrows)/2. for x in range(nrows)], hspace = 0.3
)

plot_counter = 0
logo_counter = 0
for row in range(nrows):
    for col in range(ncols):
        if row % 2 != 0:
            ax = plt.subplot(
                gspec[row, col],
                facecolor = "#EFE9E6"
            )

            if row == 1 and col == 0:
                legend = True
            else:
                legend = False
            
            plot_battery_chart(data = df, team_id=df["team_id"].iloc[plot_counter], ax = ax, legend = legend)            

            plot_counter += 1
        
        else:
            team_id = df["team_id"].iloc[logo_counter]
            team_name = df["team_name"].iloc[logo_counter]
            goal_set_pieces = df[df["team_id"] == team_id]["goals_set_pieces"].iloc[0]
            xg_set_pieces = df[df["team_id"] == team_id]["xG_set_pieces"].iloc[0]
            goals_for = df[df["team_id"] == team_id]["score_for"].iloc[0]
            xg_for = df[df["team_id"] == team_id]["xG_for"].iloc[0]


            logo_ax = plt.subplot(
                gspec[row,col],
                anchor = "NW", facecolor = "#EFE9E6"
            )
            club_icon = Image.open(urllib.request.urlopen(f"https://images.fotmob.com/image_resources/logo/teamlogo/{team_id:.0f}.png")).convert("LA")
            logo_ax.imshow(club_icon)
            logo_ax.axis("off")

            # Add the team name
            ax_text(
                x = 1.3, 
                y = 0.81,
                s = f"{team_name}",
                ax = logo_ax, 
                weight = "bold", 
                font = "Karla", 
                ha = "left", 
                size = 6, 
                annotationbbox_kw = {"xycoords":"axes fraction"}
            )

            # Add the subtitles for each side
            ax_text(
                x = 1.3,
                y = 0.32,
                s = f"<Goals: {goals_for} | From set-pieces: {goal_set_pieces}>\n<xG: {xg_for:.1f} | From set-pieces: {xg_set_pieces:.1f}>",
                highlight_textprops=[
                    {"color":"#74959A"},
                    {"color":"#495371"}
                ],
                ax = logo_ax, 
                weight = "normal", 
                font = "Karla", 
                ha = "left", 
                size = 5, 
                annotationbbox_kw = {"xycoords":"axes fraction"}
            )

            logo_counter += 1


fig_text(
    x = 0.17, y = .91, 
    s = "Which MLS sides are making the most of set-pieces?",
    va = "bottom", ha = "left",
    fontsize = 14, color = "black", font = "DM Sans", weight = "bold"
)
fig_text(
	x = 0.17, y = .895, 
    s = "Percentage of <goals> and <expected goals> created from set-pieces and corners | Data as of June 12, 2022 | viz by @sonofacorner",
    highlight_textprops=[
        {"color":"#74959A", "weight":"bold"},
        {"color":"#495371", "weight":"bold"}
    ],
	va = "bottom", ha = "left",
	fontsize = 6, color = "#4E616C", font = "Karla"
)

# # ---- The League's logo
league_icon = Image.open(urllib.request.urlopen(f"https://images.fotmob.com/image_resources/logo/leaguelogo/130.png")).convert("LA")
league_ax = fig.add_axes([0.095, 0.89, 0.05, 0.05], zorder=1)
league_ax.imshow(league_icon)
league_ax.axis("off")

plt.savefig(
	"figures/06112022_mls_set_piece.png",
	dpi = 500,
	facecolor = "#EFE9E6",
	bbox_inches="tight",
    edgecolor="none",
	transparent = False
)

plt.savefig(
	"figures/06112022_mls_set_piece_tr.png",
	dpi = 500,
	facecolor = "none",
	bbox_inches="tight",
    edgecolor="none",
	transparent = True
)